@@ -76,6 +76,7 @@ def getConstantDefaultPopulation():
         "__init__",
         "__cmp__",
         "__iter__",
+        "__loader__",
         # Nuitka specific
         "__compiled__",
         "__nuitka__",
@@ -107,6 +108,8 @@ def getConstantDefaultPopulation():
         "\\",
         "path",
         "basename",
+        "abspath",
+        "isabs",
         "exists",
         "isdir",
         "isfile",
@@ -121,7 +124,7 @@ def getConstantDefaultPopulation():
 
     if python_version >= 0x300:
         # For Python3 modules
-        result += ("__cached__", "__loader__")
+        result += ("__cached__",)
 
         # For Python3 print
         result += ("print", "end", "file")
@@ -201,6 +204,10 @@ def getConstantDefaultPopulation():
         if Options.is_debug:
             result.append("__args__")
 
+    if python_version >= 0x3B0:
+        result.append("__aenter__")
+        result.append("__aexit__")
+
     if isWin32Windows():
         result.append("fileno")
 
