@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,13 +24,14 @@ from abc import abstractmethod
 
 from nuitka.PythonVersions import needsSetLiteralReverseInsertion
 
+from .ChildrenHavingMixins import ChildHavingElementsTupleMixin
 from .ConstantRefNodes import (
     ExpressionConstantListEmptyRef,
     ExpressionConstantSetEmptyRef,
     ExpressionConstantTupleEmptyRef,
     makeConstantRefNode,
 )
-from .ExpressionBases import ExpressionChildTupleHavingBase
+from .ExpressionBases import ExpressionBase
 from .ExpressionShapeMixins import (
     ExpressionListShapeExactMixin,
     ExpressionSetShapeExactMixin,
@@ -38,65 +39,11 @@ from .ExpressionShapeMixins import (
 )
 from .IterationHandles import ListAndTupleContainerMakingIterationHandle
 from .NodeBases import SideEffectsFromChildrenMixin
-from .NodeMakingHelpers import (
-    makeStatementOnlyNodesFromExpressions,
-    wrapExpressionWithSideEffects,
-)
+from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
 
 
-class ExpressionMakeSequenceBase(
-    SideEffectsFromChildrenMixin, ExpressionChildTupleHavingBase
-):
-    __slots__ = ("sequence_kind",)
-
-    named_child = "elements"
-
-    def __init__(self, sequence_kind, elements, source_ref):
-        assert elements
-
-        for element in elements:
-            assert element.isExpression(), element
-
-        self.sequence_kind = sequence_kind.lower()
-
-        ExpressionChildTupleHavingBase.__init__(
-            self, value=tuple(elements), source_ref=source_ref
-        )
-
-    @staticmethod
-    def isExpressionMakeSequence():
-        return True
-
-    @abstractmethod
-    def getSimulator(self):
-        """The simulator for the container making, for overload."""
-
-    def computeExpression(self, trace_collection):
-        elements = self.subnode_elements
-
-        for count, element in enumerate(elements):
-            if element.willRaiseException(BaseException):
-                result = wrapExpressionWithSideEffects(
-                    side_effects=elements[:count], new_node=element, old_node=self
-                )
-
-                return result, "new_raise", "Sequence creation raises exception"
-
-        for element in elements:
-            if not element.isCompileTimeConstant():
-                return self, None, None
-
-        simulator = self.getSimulator()
-        assert simulator is not None
-
-        return trace_collection.getCompileTimeComputationResult(
-            node=self,
-            computation=lambda: simulator(
-                element.getCompileTimeConstant() for element in elements
-            ),
-            description="%s with constant arguments." % simulator.__name__.capitalize(),
-            user_provided=True,
-        )
+class ExpressionMakeSequenceMixin(object):
+    __slots__ = ()
 
     def isKnownToBeIterable(self, count):
         return count is None or count == len(self.subnode_elements)
@@ -131,7 +78,10 @@ class ExpressionMakeSequenceBase(
 
         return False
 
+    # TODO: Make this happen from auto-compute, children only side effects
     def computeExpressionDrop(self, statement, trace_collection):
+        # Virtual method overload, pylint: disable=unused-argument
+
         result = makeStatementOnlyNodesFromExpressions(
             expressions=self.subnode_elements
         )
@@ -142,13 +92,67 @@ class ExpressionMakeSequenceBase(
             result,
             "new_statements",
             """\
-Removed sequence creation for unused sequence.""",
+Removed %s creation for unused sequence."""
+            % self.getSequenceName(),
         )
 
     def onContentEscapes(self, trace_collection):
         for element in self.subnode_elements:
             element.onContentEscapes(trace_collection)
 
+    @abstractmethod
+    def getSequenceName(self):
+        """Get name for use in traces"""
+
+
+class ExpressionMakeSequenceBase(
+    SideEffectsFromChildrenMixin,
+    ExpressionMakeSequenceMixin,
+    ChildHavingElementsTupleMixin,
+    ExpressionBase,
+):
+    named_children = ("elements|tuple",)
+
+    def __init__(self, elements, source_ref):
+        assert elements
+
+        ChildHavingElementsTupleMixin.__init__(
+            self,
+            elements=elements,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+    def getSequenceName(self):
+        """Get name for use in traces"""
+        simulator = self.getSimulator()
+        return simulator.__name__.capitalize()
+
+    @staticmethod
+    def isExpressionMakeSequence():
+        return True
+
+    @abstractmethod
+    def getSimulator(self):
+        """The simulator for the container making, for overload."""
+
+    def computeExpression(self, trace_collection):
+        for element in self.subnode_elements:
+            if not element.isCompileTimeConstant():
+                return self, None, None
+
+        simulator = self.getSimulator()
+        assert simulator is not None
+
+        return trace_collection.getCompileTimeComputationResult(
+            node=self,
+            computation=lambda: simulator(
+                element.getCompileTimeConstant() for element in self.subnode_elements
+            ),
+            description="%s with constant arguments." % simulator.__name__.capitalize(),
+            user_provided=True,
+        )
+
 
 def makeExpressionMakeTuple(elements, source_ref):
     if elements:
@@ -186,7 +190,7 @@ class ExpressionMakeTuple(ExpressionTupleShapeExactMixin, ExpressionMakeSequence
 
     def __init__(self, elements, source_ref):
         ExpressionMakeSequenceBase.__init__(
-            self, sequence_kind="TUPLE", elements=elements, source_ref=source_ref
+            self, elements=elements, source_ref=source_ref
         )
 
     @staticmethod
@@ -208,8 +212,6 @@ def makeExpressionMakeList(elements, source_ref):
 
 
 def makeExpressionMakeListOrConstant(elements, user_provided, source_ref):
-    assert type(elements) is list
-
     for element in elements:
         # TODO: Compile time constant ought to be the criterion.
         if not element.isExpressionConstantRef():
@@ -230,20 +232,8 @@ def makeExpressionMakeListOrConstant(elements, user_provided, source_ref):
     return result
 
 
-class ExpressionMakeList(ExpressionListShapeExactMixin, ExpressionMakeSequenceBase):
-    kind = "EXPRESSION_MAKE_LIST"
-
-    def __init__(self, elements, source_ref):
-        ExpressionMakeSequenceBase.__init__(
-            self, sequence_kind="LIST", elements=elements, source_ref=source_ref
-        )
-
-    @staticmethod
-    def getSimulator():
-        return list
-
-    def getIterationLength(self):
-        return len(self.subnode_elements)
+class ExpressionMakeListMixin(object):
+    __slots__ = ()
 
     def computeExpressionIter1(self, iter_node, trace_collection):
         result = ExpressionMakeTuple(
@@ -261,12 +251,30 @@ Iteration over list lowered to iteration over tuple.""",
         )
 
 
+class ExpressionMakeList(
+    ExpressionListShapeExactMixin, ExpressionMakeListMixin, ExpressionMakeSequenceBase
+):
+    kind = "EXPRESSION_MAKE_LIST"
+
+    def __init__(self, elements, source_ref):
+        ExpressionMakeSequenceBase.__init__(
+            self, elements=elements, source_ref=source_ref
+        )
+
+    @staticmethod
+    def getSimulator():
+        return list
+
+    def getIterationLength(self):
+        return len(self.subnode_elements)
+
+
 class ExpressionMakeSet(ExpressionSetShapeExactMixin, ExpressionMakeSequenceBase):
     kind = "EXPRESSION_MAKE_SET"
 
     def __init__(self, elements, source_ref):
         ExpressionMakeSequenceBase.__init__(
-            self, sequence_kind="SET", elements=elements, source_ref=source_ref
+            self, elements=elements, source_ref=source_ref
         )
 
     @staticmethod
@@ -287,6 +295,39 @@ class ExpressionMakeSet(ExpressionSetShapeExactMixin, ExpressionMakeSequenceBase
         # Hashing and equality may consume elements of the produced set.
         return 1
 
+    def computeExpression(self, trace_collection):
+        # For sets, we need to consider hashing
+        are_constants = True
+        are_hashable = True
+
+        for element in self.subnode_elements:
+            if are_constants and not element.isCompileTimeConstant():
+                are_constants = False
+
+            if are_hashable and not element.isKnownToBeHashable():
+                are_hashable = False
+
+            if not are_hashable and not are_constants:
+                break
+
+        if not are_constants:
+            if not are_hashable:
+                trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        simulator = self.getSimulator()
+        assert simulator is not None
+
+        return trace_collection.getCompileTimeComputationResult(
+            node=self,
+            computation=lambda: simulator(
+                element.getCompileTimeConstant() for element in self.subnode_elements
+            ),
+            description="%s with constant arguments." % simulator.__name__.capitalize(),
+            user_provided=True,
+        )
+
     def mayRaiseException(self, exception_type):
         for element in self.subnode_elements:
             if not element.isKnownToBeHashable():
