@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,10 +31,17 @@ from .BuiltinRefNodes import (
     ExpressionBuiltinRef,
     makeExpressionBuiltinRef,
 )
-from .ExpressionBases import (
-    ExpressionBuiltinSingleArgBase,
-    ExpressionChildrenHavingBase,
+from .ChildrenHavingMixins import (
+    ChildHavingClsMixin,
+    ChildrenExpressionBuiltinIssubclassMixin,
+    ChildrenExpressionBuiltinSuper0Mixin,
+    ChildrenExpressionBuiltinSuper1Mixin,
+    ChildrenExpressionBuiltinSuper2Mixin,
+    ChildrenHavingInstanceClassesMixin,
 )
+from .ExpressionBases import ExpressionBase, ExpressionBuiltinSingleArgBase
+from .ExpressionShapeMixins import ExpressionBoolShapeExactMixin
+from .NodeBases import SideEffectsFromChildrenMixin
 from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 from .shapes.BuiltinTypeShapes import tshape_type
 
@@ -119,20 +126,44 @@ Removed type taking for unused result.""",
         return self.subnode_value.mayHaveSideEffects()
 
 
-class ExpressionBuiltinSuper2(ExpressionChildrenHavingBase):
-    """ Two arguments form of super. """
+class ExpressionBuiltinSuper1(ChildrenExpressionBuiltinSuper1Mixin, ExpressionBase):
+    """Two arguments form of super."""
+
+    kind = "EXPRESSION_BUILTIN_SUPER1"
+
+    named_children = ("type_arg",)
+
+    def __init__(self, type_arg, source_ref):
+        ChildrenExpressionBuiltinSuper1Mixin.__init__(
+            self,
+            type_arg=type_arg,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+    def computeExpression(self, trace_collection):
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        # TODO: Quite some cases should be possible to predict.
+        return self, None, None
+
+
+class ExpressionBuiltinSuper2(ChildrenExpressionBuiltinSuper2Mixin, ExpressionBase):
+    """Two arguments form of super."""
 
     kind = "EXPRESSION_BUILTIN_SUPER2"
 
     named_children = ("type_arg", "object_arg")
 
     def __init__(self, type_arg, object_arg, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenExpressionBuiltinSuper2Mixin.__init__(
             self,
-            values={"type_arg": type_arg, "object_arg": object_arg},
-            source_ref=source_ref,
+            type_arg=type_arg,
+            object_arg=object_arg,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
@@ -140,20 +171,22 @@ class ExpressionBuiltinSuper2(ExpressionChildrenHavingBase):
         return self, None, None
 
 
-class ExpressionBuiltinSuper0(ExpressionChildrenHavingBase):
-    """ Python3 form of super, arguments determined from cells and function arguments. """
+class ExpressionBuiltinSuper0(ChildrenExpressionBuiltinSuper0Mixin, ExpressionBase):
+    """Python3 form of super, arguments determined from cells and function arguments."""
 
     kind = "EXPRESSION_BUILTIN_SUPER0"
 
     named_children = ("type_arg", "object_arg")
 
     def __init__(self, type_arg, object_arg, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenExpressionBuiltinSuper0Mixin.__init__(
             self,
-            values={"type_arg": type_arg, "object_arg": object_arg},
-            source_ref=source_ref,
+            type_arg=type_arg,
+            object_arg=object_arg,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
@@ -161,18 +194,20 @@ class ExpressionBuiltinSuper0(ExpressionChildrenHavingBase):
         return self, None, None
 
 
-class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
+class ExpressionBuiltinIsinstance(ChildrenHavingInstanceClassesMixin, ExpressionBase):
     kind = "EXPRESSION_BUILTIN_ISINSTANCE"
 
     named_children = ("instance", "classes")
 
     def __init__(self, instance, classes, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenHavingInstanceClassesMixin.__init__(
             self,
-            values={"instance": instance, "classes": classes},
-            source_ref=source_ref,
+            instance=instance,
+            classes=classes,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         # TODO: Quite some cases should be possible to predict.
 
@@ -202,18 +237,22 @@ class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
         )
 
 
-class ExpressionBuiltinIssubclass(ExpressionChildrenHavingBase):
+class ExpressionBuiltinIssubclass(
+    ChildrenExpressionBuiltinIssubclassMixin, ExpressionBase
+):
     kind = "EXPRESSION_BUILTIN_ISSUBCLASS"
 
     named_children = ("cls", "classes")
 
     def __init__(self, cls, classes, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenExpressionBuiltinIssubclassMixin.__init__(
             self,
-            values={"cls": cls, "classes": classes},
-            source_ref=source_ref,
+            cls=cls,
+            classes=classes,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         # TODO: Quite some cases should be possible to predict.
 
@@ -241,3 +280,24 @@ class ExpressionBuiltinIssubclass(ExpressionChildrenHavingBase):
             ),
             description="Built-in call to 'issubclass' computed.",
         )
+
+
+class ExpressionTypeCheck(
+    ExpressionBoolShapeExactMixin,
+    SideEffectsFromChildrenMixin,
+    ChildHavingClsMixin,
+    ExpressionBase,
+):
+    kind = "EXPRESSION_TYPE_CHECK"
+
+    named_children = ("cls",)
+
+    def __init__(self, cls, source_ref):
+        ChildHavingClsMixin.__init__(self, cls=cls)
+
+        ExpressionBase.__init__(self, source_ref)
+
+    def computeExpression(self, trace_collection):
+        # TODO: Quite some cases should be possible to predict, but I am not aware of
+        # 100% true Python equivalent at this time.
+        return self, None, None
