@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -40,7 +40,7 @@ from nuitka.plugins.PluginBase import NuitkaPluginBase
 
 class NuitkaPluginPylintEclipseAnnotations(NuitkaPluginBase):
     plugin_name = "pylint-warnings"
-    plugin_desc = "Support PyLint / PyDev linting source markers"
+    plugin_desc = "Support PyLint / PyDev linting source markers."
 
     def __init__(self):
         self.line_annotations = {}
