@@ -41,7 +41,7 @@ class NuitkaPluginGlfw(NuitkaPluginBase):
 
     # TODO: Maybe rename to opengl maybe
     plugin_name = "glfw"  # Nuitka knows us by this name
-    plugin_desc = "Required for OpenGL and glfw in standalone mode"
+    plugin_desc = "Required for OpenGL and 'glfw' package in standalone mode."
 
     @staticmethod
     def isAlwaysEnabled():
@@ -123,7 +123,7 @@ class NuitkaPluginGlfw(NuitkaPluginBase):
 
             code = r"""
 import os
-os.environ["PYGLFW_LIBRARY"] = os.path.join(__nuitka_binary_dir, "glfw", %r)
+os.environ["PYGLFW_LIBRARY"] = os.path.join(__nuitka_binary_dir, "glfw", "%s")
 """ % os.path.basename(
                 dll_filename
             )
