@@ -27,11 +27,13 @@ import subprocess
 import sys
 from optparse import OptionParser
 
+from nuitka.PythonVersions import getTestExecutionPythonVersions
 from nuitka.tools.Basics import goHome
 from nuitka.tools.testing.Common import (
     getInstalledPythonVersion,
     getTempDir,
     my_print,
+    test_logger,
     withExtendedExtraOptions,
 )
 from nuitka.utils.Execution import check_call, check_output, getNullOutput
@@ -254,6 +256,16 @@ The standard CPython3.10 test suite. Execute this for all corner cases to be
 covered. With Python 2.x these are not run. Default is %default.""",
     )
 
+    parser.add_option(
+        "--skip-cpython311-tests",
+        action="store_false",
+        dest="cpython311",
+        default=True,
+        help="""\
+The standard CPython3.11 test suite. Execute this for all corner cases to be
+covered. With Python 2.x these are not run. Default is %default.""",
+    )
+
     parser.add_option(
         "--skip-other-cpython-tests",
         action="store_true",
@@ -374,6 +386,15 @@ Do not use Python3.9 even if available on the system. Default is %default.""",
 Do not use Python3.10 even if available on the system. Default is %default.""",
     )
 
+    parser.add_option(
+        "--no-python3.11",
+        action="store_true",
+        dest="no311",
+        default=False,
+        help="""\
+Do not use Python3.11 even if available on the system. Default is %default.""",
+    )
+
     parser.add_option(
         "--coverage",
         action="store_true",
@@ -438,6 +459,8 @@ Enforce the use of MinGW64 on Windows. Defaults to off.""",
             options.no39 = True
         if sys.version_info[0:2] != (3, 10):
             options.no310 = True
+        if sys.version_info[0:2] != (3, 11):
+            options.no311 = True
 
     if options.cpython_no_other:
         if sys.version_info[0:2] != (2, 6):
@@ -462,6 +485,8 @@ Enforce the use of MinGW64 on Windows. Defaults to off.""",
             options.cpython39 = False
         if sys.version_info[0:2] != (3, 10):
             options.cpython310 = False
+        if sys.version_info[0:2] != (3, 11):
+            options.cpython311 = False
 
     if options.cpython_none:
         options.cpython26 = False
@@ -475,6 +500,7 @@ Enforce the use of MinGW64 on Windows. Defaults to off.""",
         options.cpython38 = False
         options.cpython39 = False
         options.cpython310 = False
+        options.cpython311 = False
 
     if options.coverage and os.path.exists(".coverage"):
         os.unlink(".coverage")
@@ -508,8 +534,8 @@ def publishCoverageData():
         if sys.version_info > (3,):
             nuitka_id = nuitka_id.decode()
 
-        data_file.write("NUITKA_SOURCE_DIR=%r\n" % source_dir)
-        data_file.write("NUITKA_COMMIT=%r\n" % nuitka_id)
+        data_file.write("NUITKA_SOURCE_DIR='%s'\n" % source_dir)
+        data_file.write("NUITKA_COMMIT='%s'\n" % nuitka_id)
 
     copyToGlobalCoverageData("data.coverage", "meta.coverage." + suffix)
 
@@ -580,6 +606,8 @@ def main():
             return False
         if command == "python3.10" and options.no310:
             return False
+        if command == "python3.11" and options.no311:
+            return False
 
         # Shortcuts for python versions, also needed for Windows as it won't have
         # the version number in the Python binaries at all.
@@ -603,6 +631,8 @@ def main():
             return True
         if command == "python3.10" and sys.version_info[0:2] == (3, 10):
             return True
+        if command == "python3.11" and sys.version_info[0:2] == (3, 11):
+            return True
 
         path = os.environ["PATH"]
 
@@ -909,18 +939,22 @@ def main():
                     else:
                         my_print("The CPython3.10 tests are not present, not run.")
 
-    assert (
-        checkExecutableCommand("python2.6")
-        or checkExecutableCommand("python2.7")
-        or checkExecutableCommand("python3.3")
-        or checkExecutableCommand("python3.4")
-        or checkExecutableCommand("python3.5")
-        or checkExecutableCommand("python3.6")
-        or checkExecutableCommand("python3.7")
-        or checkExecutableCommand("python3.8")
-        or checkExecutableCommand("python3.9")
-        or checkExecutableCommand("python3.10")
-    )
+            # Running the Python 3.11 test suite only with CPython3.x.
+            if not use_python.startswith("python2"):
+                if options.cpython311:
+                    if os.path.exists("./tests/CPython311/run_all.py"):
+                        with withExtendedExtraOptions(
+                            *getExtraFlags(where, "311tests", flags)
+                        ):
+                            executeSubTest("./tests/CPython311/run_all.py search")
+                    else:
+                        my_print("The CPython3.11 tests are not present, not run.")
+
+    if not any(
+        checkExecutableCommand("python%s" % python_version)
+        for python_version in getTestExecutionPythonVersions()
+    ):
+        test_logger.sysexit("No Python usable for testing found installed")
 
     if options.debug:
         if checkExecutableCommand("python2.6"):
@@ -983,6 +1017,11 @@ def main():
     else:
         my_print("Cannot execute tests with Python 3.10, disabled or not installed.")
 
+    if checkExecutableCommand("python3.11"):
+        execute_tests("python3.11-nodebug", "python3.11", "")
+    else:
+        my_print("Cannot execute tests with Python 3.11, disabled or not installed.")
+
     if options.coverage:
         publishCoverageData()
 
