@@ -168,8 +168,7 @@ def buildComplexComparisonNode(provider, left, rights, comparators, source_ref):
                 )
             )
 
-    outline_body.setChild(
-        "body",
+    outline_body.setChildBody(
         makeStatementsSequenceFromStatement(
             statement=makeTryFinallyStatement(
                 provider=outline_body,
@@ -177,7 +176,7 @@ def buildComplexComparisonNode(provider, left, rights, comparators, source_ref):
                 final=final,
                 source_ref=source_ref,
             )
-        ),
+        )
     )
 
     return outline_body
