@@ -33,6 +33,7 @@ from nuitka.nodes.DictionaryNodes import makeExpressionMakeDict
 from nuitka.nodes.ExceptionNodes import StatementReraiseException
 from nuitka.nodes.FrameNodes import (
     StatementsFrameAsyncgen,
+    StatementsFrameClass,
     StatementsFrameCoroutine,
     StatementsFrameFunction,
     StatementsFrameGenerator,
@@ -72,7 +73,7 @@ def extractDocFromBody(node):
                 doc = body[0].value.value
             body = body[1:]
 
-        if Options.hasPythonFlagNoDocstrings():
+        if Options.hasPythonFlagNoDocStrings():
             doc = None
 
     return body, doc
@@ -367,6 +368,26 @@ def buildNodeList(provider, nodes, source_ref, allow_none=False):
         return []
 
 
+def buildNodeTuple(provider, nodes, source_ref, allow_none=False):
+    if nodes is not None:
+        result = []
+
+        for node in nodes:
+            if hasattr(node, "lineno"):
+                node_source_ref = source_ref.atLineNumber(node.lineno)
+            else:
+                node_source_ref = source_ref
+
+            entry = buildNode(provider, node, node_source_ref, allow_none)
+
+            if entry is not None:
+                result.append(entry)
+
+        return tuple(result)
+    else:
+        return ()
+
+
 _host_node = None
 
 
@@ -422,9 +443,10 @@ def makeModuleFrame(module, statements, source_ref):
             code_name = "<module %s>" % module.getFullName()
 
     return StatementsFrameModule(
-        statements=statements,
+        statements=tuple(statements),
         code_object=CodeObjectSpec(
             co_name=code_name,
+            co_qualname=code_name,
             co_kind="Module",
             co_varnames=(),
             co_freevars=(),
@@ -477,10 +499,17 @@ def buildFrameNode(provider, nodes, code_object, source_ref):
     if provider.isExpressionOutlineFunction():
         provider = provider.getParentVariableProvider()
 
-    if provider.isExpressionFunctionBody() or provider.isExpressionClassBody():
+    if provider.isExpressionFunctionBody():
         result = StatementsFrameFunction(
             statements=statements, code_object=code_object, source_ref=source_ref
         )
+    elif provider.isExpressionClassBodyBase():
+        result = StatementsFrameClass(
+            statements=statements,
+            code_object=code_object,
+            locals_scope=provider.getLocalsScope(),
+            source_ref=source_ref,
+        )
     elif provider.isExpressionGeneratorObjectBody():
         result = StatementsFrameGenerator(
             statements=statements, code_object=code_object, source_ref=source_ref
