@@ -37,6 +37,7 @@ from .Execution import (
 )
 from .FileOperations import (
     addFileExecutablePermission,
+    changeFilenameExtension,
     copyFile,
     getFileList,
     makeContainingPath,
@@ -170,10 +171,10 @@ def _getDLLVersionWindows(filename):
 
     if type(filename) is unicode:
         GetFileVersionInfoSizeW = ctypes.windll.version.GetFileVersionInfoSizeW
-        GetFileVersionInfoSizeW.argtypes = [
+        GetFileVersionInfoSizeW.argtypes = (
             ctypes.wintypes.LPCWSTR,
             ctypes.wintypes.LPDWORD,
-        ]
+        )
         GetFileVersionInfoSizeW.restype = ctypes.wintypes.HANDLE
         size = GetFileVersionInfoSizeW(filename, None)
     else:
@@ -189,12 +190,12 @@ def _getDLLVersionWindows(filename):
     if type(filename) is unicode:
         # Python3 needs our help here.
         GetFileVersionInfo = ctypes.windll.version.GetFileVersionInfoW
-        GetFileVersionInfo.argtypes = [
+        GetFileVersionInfo.argtypes = (
             ctypes.wintypes.LPCWSTR,
             ctypes.wintypes.DWORD,
             ctypes.wintypes.DWORD,
             ctypes.wintypes.LPVOID,
-        ]
+        )
         GetFileVersionInfo.restype = ctypes.wintypes.BOOL
 
     else:
@@ -207,12 +208,12 @@ def _getDLLVersionWindows(filename):
 
     # Look for codepages
     VerQueryValueA = ctypes.windll.version.VerQueryValueA
-    VerQueryValueA.argtypes = [
+    VerQueryValueA.argtypes = (
         ctypes.wintypes.LPCVOID,
         ctypes.wintypes.LPCSTR,
         ctypes.wintypes.LPVOID,
         ctypes.POINTER(ctypes.c_uint32),
-    ]
+    )
     VerQueryValueA.restype = ctypes.wintypes.BOOL
 
     file_info = ctypes.POINTER(VsFixedFileInfoStructure)()
@@ -352,6 +353,11 @@ def _filterPatchelfErrorOutput(stderr):
     return None, stderr
 
 
+_patchelf_usage = """\
+Error, needs 'patchelf' on your system, to modify 'RPATH' settings that \
+need to be updated."""
+
+
 def _setSharedLibraryRPATHElf(filename, rpath):
     # patchelf --set-rpath "$ORIGIN/path/to/library" <executable>
     with withEnvironmentVarOverridden("LANG", "C"):
@@ -359,9 +365,7 @@ def _setSharedLibraryRPATHElf(filename, rpath):
             logger=postprocessing_logger,
             command=("patchelf", "--set-rpath", rpath, filename),
             stderr_filter=_filterPatchelfErrorOutput,
-            absence_message="""\
-Error, needs 'patchelf' on your system, due to 'RPATH' settings that need to be
-set.""",
+            absence_message=_patchelf_usage,
         )
 
 
@@ -636,6 +640,17 @@ def copyDllFile(source_path, dist_dir, dest_path, executable):
         rpath = os.path.join("$ORIGIN", *([".."] * count))
         setSharedLibraryRPATH(target_filename, rpath)
 
+    if isWin32Windows() and Options.is_debug:
+        pdb_filename = changeFilenameExtension(path=source_path, extension=".pdb")
+
+        if os.path.exists(pdb_filename):
+            copyFile(
+                source_path=pdb_filename,
+                dest_path=changeFilenameExtension(
+                    path=target_filename, extension=".pdb"
+                ),
+            )
+
     if executable:
         addFileExecutablePermission(target_filename)
 
@@ -684,12 +699,12 @@ def _getWindowsRunningProcessModuleHandles():
         EnumProcessModulesProc = ctypes.windll.kernel32.EnumProcessModules
 
     EnumProcessModulesProc.restype = ctypes.wintypes.BOOL
-    EnumProcessModulesProc.argtypes = [
+    EnumProcessModulesProc.argtypes = (
         ctypes.wintypes.HANDLE,
         ctypes.POINTER(ctypes.wintypes.HANDLE),
         ctypes.wintypes.LONG,
         ctypes.POINTER(ctypes.wintypes.ULONG),
-    ]
+    )
 
     # Very unlikely that this is not sufficient for CPython.
     handles = (ctypes.wintypes.HANDLE * 1024)()
@@ -715,8 +730,27 @@ def getWindowsRunningProcessDLLPaths():
     result = OrderedDict()
 
     for handle in _getWindowsRunningProcessModuleHandles():
-        filename = getWindowsRunningProcessModuleFilename(handle)
+        try:
+            filename = getWindowsRunningProcessModuleFilename(handle)
+        except WindowsError:
+            continue
 
         result[os.path.basename(filename)] = filename
 
     return result
+
+
+def convertRPathToRunPath(filename):
+    """Change a DT_RPATH to DT_RUNPATH
+
+    On Android this seems required, because the linker doesn't support the one
+    created by default.
+    """
+
+    with withEnvironmentVarOverridden("LANG", "C"):
+        executeToolChecked(
+            logger=postprocessing_logger,
+            command=("patchelf", "--shrink-rpath", filename),
+            stderr_filter=_filterPatchelfErrorOutput,
+            absence_message=_patchelf_usage,
+        )
