@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -17,24 +17,26 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-
 # Format string with local compile time constants fully optimized:
 def f():
     a = "1"
     b = "2"
 
-    return f'r:{a!r} and a:{a!a} and s:{a!s}'
+    return f"r:{a!r} and a:{a!a} and s:{a!s}"
+
 
 print(f())
 
 # Empty f string fully optimized:
 def f():
-    return f''
+    return f""
+
 
 print(f())
 
 # Text only f string fully optimized:
 def f():
-    return f'some_text'
+    return f"some_text"
+
 
 print(f())
