@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,6 +17,6 @@
 #
 print("Child1:", __name__)
 
-from . import Child2
+from . import Child2  # isort:skip
 
 print("Value of Child2", Child2.__name__)
