@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -33,7 +33,10 @@ def displayDict(d):
     del d["__builtins__"]
 
     if "__loader__" in d:
-        d["__loader__"] = "<__loader__ removed>"
+        if str is bytes:
+            del d["__loader__"]
+        else:
+            d["__loader__"] = "<__loader__ removed>"
 
     if "__file__" in d:
         d["__file__"] = "<__file__ removed>"
