@@ -15,5 +15,4 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-
-x = (yield from range(8))
+x = yield from range(8)
